import java.util.Stack;

import ch.usi.dag.disl.annotation.After;
import ch.usi.dag.disl.annotation.Before;
import ch.usi.dag.disl.annotation.SyntheticLocal;
import ch.usi.dag.disl.annotation.ThreadLocal;
import ch.usi.dag.disl.marker.BodyMarker;
import ch.usi.dag.disl.staticcontext.MethodStaticContext;

public class CallStackReification {

	@ThreadLocal
	static Stack<String> cs; // call stack

	@SyntheticLocal
	static long entryTime;

	@Before(marker = BodyMarker.class, order = 1000)
	static void enterCS(MethodStaticContext msc) {

		// create call stack
		if (cs == null) {
			cs = new Stack<String>();
		}

		// maintain call stack
		cs.push(msc.thisMethodFullName());
	}

	@Before(marker = BodyMarker.class)
	static void onMethodEntry() {

		// sample entry time
		entryTime = System.nanoTime();
	}

	@After(marker = BodyMarker.class)
	static void onMethodExit(MethodStaticContext msc) {

		// print method duration
		System.out.println("Instrumentation: " + msc.thisMethodFullName()
				+ " duration " + (System.nanoTime() - entryTime) + " ns");

		// print call stack (without the current method)
		for(int i = cs.size() - 2; i >= 0; --i) {
			String fromMethod = cs.get(i);
			System.out.println("Instrumentation:    invoked from: " + fromMethod);
		}
	}

	@After(marker = BodyMarker.class, order = 1000)
	static void exitCS() {

		// maintain call stack
		String method = cs.pop();
	}

}
